@@ -1,21 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Function;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.60_004';
+our $VERSION = '0.38';
 
 use base qw( IO::Async::Notifier );
 use IO::Async::Timer::Countdown;
 
 use Carp;
 
-use Storable qw( freeze );
+use Storable qw( freeze thaw );
 
 =head1 NAME
 
@@ -26,7 +26,7 @@ C<IO::Async::Function> - call a function asynchronously
  use IO::Async::Function;
 
  use IO::Async::Loop;
- my $loop = IO::Async::Loop->new;
+ my $loop = IO::Async::Loop->new();
 
  my $function = IO::Async::Function->new(
     code => sub {
@@ -48,7 +48,7 @@ C<IO::Async::Function> - call a function asynchronously
     },
  );
 
- $loop->run;
+ $loop->loop_forever;
 
 =head1 DESCRIPTION
 
@@ -59,7 +59,7 @@ passing in arguments, and invoking a continuation in the main process when the
 function returns.
 
 The object represents the function code itself, rather than one specific
-invocation of it. It can be called multiple times, by the C<call> method.
+invocation of it. It can be called multiple times, by the C<call()> method.
 Multiple outstanding invocations can be called; they will be dispatched in
 the order they were queued. If only one worker process is used then results
 will be returned in the order they were called. If multiple are used, then
@@ -73,8 +73,11 @@ modifications to the state of the parent program. Therefore, all the data
 required to perform its task must be represented in the call arguments, and
 all of the result must be represented in the return values.
 
-The Function object is implemented using an L<IO::Async::Routine> with two
-L<IO::Async::Channel> objects to pass calls into and results out from it.
+The arguments and return values are passed to it over a file handle, using
+L<Storable>. This can cope with most kinds of Perl data, including plain
+numbers and strings, references to hashes or arrays, and self-referential or
+even cyclic data structures. Note however that passing C<CODE> references or
+IO handles is not supported.
 
 The C<IO::Async> framework generally provides mechanisms for multiplexing IO
 tasks between different handles, so there aren't many occasions when such an
@@ -86,7 +89,7 @@ are:
 =item 1.
 
 When a large amount of computationally-intensive work needs to be performed
-(for example, the C<is_prime> test in the example in the C<SYNOPSIS>).
+(for example, the C<is_prime()> test in the example in the C<SYNOPSIS>).
 
 =item 2.
 
@@ -96,11 +99,6 @@ C<IO::Async::Resolver>.
 
 =back
 
-This object is ideal for representing "pure" functions; that is, blocks of
-code which have no stateful effect on the process, and whose result depends
-only on the arguments passed in. For a more general co-routine ability, see
-also L<IO::Async::Routine>.
-
 =cut
 
 =head1 PARAMETERS
@@ -113,11 +111,6 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 The body of the function to execute.
 
-=item model => "spawn" | "thread"
-
-Optional. Requests a specific C<IO::Async::Routine> model. If not supplied,
-leaves the default choice up to Routine.
-
 =item min_workers => INT
 
 =item max_workers => INT
@@ -126,12 +119,6 @@ The lower and upper bounds of worker processes to try to keep running. The
 actual number running at any time will be kept somewhere between these bounds
 according to load.
 
-=item max_worker_calls => INT
-
-Optional. If provided, stop a worker process after it has processed this
-number of calls. (New workers may be started to replace stopped ones, within
-the bounds given above).
-
 =item idle_timeout => NUM
 
 Optional. If provided, idle worker processes will be shut down after this
@@ -161,7 +148,7 @@ sub _init
    $self->{min_workers} = 1;
    $self->{max_workers} = 8;
 
-   $self->{workers} = {}; # {$id} => IaFunction:Worker
+   $self->{workers} = {};
 
    $self->{pending_queue} = [];
 }
@@ -172,7 +159,7 @@ sub configure
    my %params = @_;
 
    my %worker_params;
-   foreach (qw( model exit_on_die max_worker_calls )) {
+   foreach (qw( exit_on_die )) {
       $self->{$_} = $worker_params{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -194,17 +181,17 @@ sub configure
          $self->{idle_timer} = IO::Async::Timer::Countdown->new(
             delay => $timeout,
             on_expire => $self->_capture_weakself( sub {
-               my $self = shift or return;
+               my $self = shift;
                my $workers = $self->{workers};
 
                # Shut down atmost one idle worker, starting from the highest
-               # ID. Since we search from lowest to assign work, this tries
+               # PID. Since we search from lowest to assign work, this tries
                # to ensure we'll shut down the least useful ones first,
                # keeping more useful ones in memory (page/cache warmth, etc..)
-               foreach my $id ( reverse sort keys %$workers ) {
-                  next if $workers->{$id}{busy};
+               foreach my $pid ( reverse sort keys %$workers ) {
+                  next if $workers->{$pid}{busy};
 
-                  $workers->{$id}->stop;
+                  $workers->{$pid}->stop;
                   last;
                }
 
@@ -229,7 +216,7 @@ sub configure
 
    $self->SUPER::configure( %params );
 
-   if( $need_restart and $self->loop ) {
+   if( $need_restart and $self->get_loop ) {
       $self->stop;
       $self->start;
    }
@@ -279,26 +266,11 @@ sub stop
 {
    my $self = shift;
 
-   $self->{stopping} = 1;
    foreach my $worker ( $self->_worker_objects ) {
       $worker->stop;
    }
 }
 
-=head2 $function->restart
-
-Gracefully stop and restart all the worker processes. 
-
-=cut
-
-sub restart
-{
-   my $self = shift;
-
-   $self->stop;
-   $self->start;
-}
-
 =head2 $function->call( %params )
 
 Schedules an invocation of the contained function to be executed on one of the
@@ -338,11 +310,6 @@ circumstances given above. They will be called directly, without the leading
 
 =back
 
-=head2 $future = $function->call( %params )
-
-When returning a future, the C<on_result>, C<on_return> and C<on_error>
-continuations are optional.
-
 =cut
 
 sub call
@@ -351,28 +318,17 @@ sub call
    my %params = @_;
 
    # TODO: possibly just queue this?
-   $self->loop or croak "Cannot ->call on a Function not yet in a Loop";
+   $self->get_loop or croak "Cannot ->call on a Function not yet in a Loop";
 
    my $args = delete $params{args};
    ref $args eq "ARRAY" or croak "Expected 'args' to be an array";
 
-   my $future = $self->loop->new_future;
+   my $request = freeze( $args );
 
+   my $on_result;
    if( defined $params{on_result} ) {
-      my $on_result = delete $params{on_result};
+      $on_result = delete $params{on_result};
       ref $on_result or croak "Expected 'on_result' to be a reference";
-
-      $future->on_done( $self->_capture_weakself( sub {
-         my $self = shift or return;
-         $self->debug_printf( "CONT on_return" );
-         $on_result->( return => @_ );
-      } ) );
-      $future->on_fail( $self->_capture_weakself( sub {
-         my $self = shift or return;
-         my ( $err, @values ) = @_;
-         $self->debug_printf( "CONT on_error" );
-         $on_result->( error => @values );
-      } ) );
    }
    elsif( defined $params{on_return} and defined $params{on_error} ) {
       my $on_return = delete $params{on_return};
@@ -380,33 +336,24 @@ sub call
       my $on_error  = delete $params{on_error};
       ref $on_error or croak "Expected 'on_error' to be a reference";
 
-      $future->on_done( $self->_capture_weakself( sub {
-         my $self = shift or return;
-         $self->debug_printf( "CONT on_return" );
-         $on_return->( @_ );
-      } ) );
-      $future->on_fail( $self->_capture_weakself( sub {
-         my $self = shift or return;
-         my ( $err, @values ) = @_;
-         $self->debug_printf( "CONT on_error" );
-         $on_error->( @values );
-      } ) );
+      $on_result = sub {
+         my $result = shift;
+         $on_return->( @_ ) if $result eq "return";
+         $on_error->( @_ )  if $result eq "error";
+      };
    }
-   elsif( !defined wantarray ) {
-      croak "Expected either 'on_result' or 'on_return' and 'on_error' keys, or to return a Future";
+   else {
+      croak "Expected either 'on_result' or 'on_return' and 'on_error' keys";
    }
 
    my $worker = $self->_get_worker;
 
    if( !$worker ) {
-      my $request = freeze( $args );
-      push @{ $self->{pending_queue} }, [ $request, $future ];
-      return $future;
+      push @{ $self->{pending_queue} }, [ $request, $on_result ];
+      return;
    }
 
-   $self->_call_worker( $worker, args => $args, $future );
-
-   return $future;
+   $self->_call_worker( $worker, $request, $on_result );
 }
 
 sub _worker_objects
@@ -456,14 +403,15 @@ sub _new_worker
    my $self = shift;
 
    my $worker = IO::Async::Function::Worker->new(
-      ( map { $_ => $self->{$_} } qw( model code setup exit_on_die ) ),
-      max_calls => $self->{max_worker_calls},
+      ( map { $_ => $self->{$_} } qw( code setup exit_on_die ) ),
 
       on_finish => $self->_capture_weakself( sub {
-         my $self = shift or return;
+         my $self = shift;
          my ( $worker ) = @_;
 
-         return if $self->{stopping};
+         if( @{ $worker->{on_result_queue} } ) {
+            print STDERR "TODO: on_result_queue to be flushed\n";
+         }
 
          $self->_new_worker if $self->workers < $self->{min_workers};
 
@@ -473,7 +421,7 @@ sub _new_worker
 
    $self->add_child( $worker );
 
-   return $self->{workers}{$worker->id} = $worker;
+   return $self->{workers}{$worker->pid} = $worker;
 }
 
 sub _get_worker
@@ -494,9 +442,9 @@ sub _get_worker
 sub _call_worker
 {
    my $self = shift;
-   my ( $worker, $type, $args, $future ) = @_;
+   my ( $worker, $request, $on_result ) = @_;
 
-   $worker->call( $type, $args, $future );
+   $worker->call( $request, $on_result );
 
    if( $self->workers_idle == 0 ) {
       $self->{idle_timer}->stop if $self->{idle_timer};
@@ -509,7 +457,7 @@ sub _dispatch_pending
 
    if( my $next = shift @{ $self->{pending_queue} } ) {
       my $worker = $self->_get_worker or return;
-      $self->_call_worker( $worker, frozen => @$next );
+      $self->_call_worker( $worker, @$next );
    }
    elsif( $self->workers_idle > $self->{min_workers} ) {
       $self->{idle_timer}->start if $self->{idle_timer} and !$self->{idle_timer}->is_running;
@@ -519,122 +467,145 @@ sub _dispatch_pending
 package # hide from indexer
    IO::Async::Function::Worker;
 
-use base qw( IO::Async::Routine );
+use base qw( IO::Async::Process );
 
-use IO::Async::Channel;
+use Storable qw( freeze thaw );
 
-sub new
+sub _read_exactly
 {
-   my $class = shift;
-   my %params = @_;
+   $_[1] = "";
 
-   my $arg_channel = IO::Async::Channel->new;
-   my $ret_channel = IO::Async::Channel->new;
+   while( length $_[1] < $_[2] ) {
+      my $n = read( $_[0], $_[1], $_[2]-length $_[1], length $_[1] );
+      defined $n or return undef;
+      $n or die "EXIT";
+   }
+}
+
+sub _init
+{
+   my $worker = shift;
+   my ( $params ) = @_;
+
+   $worker->{on_result_queue} = \my @on_result_queue;
+
+   my $code = $params->{code};
+   $params->{code} = sub {
+      while(1) {
+         my $n = _read_exactly( \*STDIN, my $lenbuffer, 4 );
+         defined $n or die "Cannot read - $!";
+
+         my $len = unpack( "I", $lenbuffer );
+
+         $n = _read_exactly( \*STDIN, my $record, $len );
+         defined $n or die "Cannot read - $!";
+
+         my $args = thaw( $record );
 
-   my $code = delete $params{code};
-   $params{code} = sub {
-      while( my $args = $arg_channel->recv ) {
          my @ret;
          my $ok = eval { @ret = $code->( @$args ); 1 };
 
          if( $ok ) {
-            $ret_channel->send( [ r => @ret ] );
+            unshift @ret, "r";
          }
          else {
-            $ret_channel->send( [ e => "$@" ] );
+            @ret = ( "e", "$@" );
          }
+
+         my $result = freeze( \@ret );
+         print STDOUT pack("I", length $result) . $result;
       }
    };
 
-   my $worker = $class->SUPER::new(
-      %params,
-      channels_in  => [ $arg_channel ],
-      channels_out => [ $ret_channel ],
-   );
+   $params->{stdin}  = { via => "pipe_write" };
+   $params->{stdout} = {
+      on_read => sub {
+         my ( $stream, $buffref, $eof ) = @_;
 
-   $worker->{arg_channel} = $arg_channel;
-   $worker->{ret_channel} = $ret_channel;
+         if( $eof ) {
+            my $on_result = shift @on_result_queue;
+            $on_result->( "eof" ) if $on_result;
+            return;
+         }
 
-   return $worker;
+         return 0 unless length( $$buffref ) >= 4;
+         my $len = unpack( "I", $$buffref );
+         return 0 unless length( $$buffref ) >= 4 + $len;
+
+         my $record = thaw( substr( $$buffref, 4, $len ) );
+         substr( $$buffref, 0, 4 + $len ) = "";
+
+         (shift @on_result_queue)->( @$record );
+
+         return 1;
+      },
+   };
+
+   $worker->SUPER::_init( $params );
 }
 
 sub configure
 {
-   my $self = shift;
+   my $worker = shift;
    my %params = @_;
 
-   exists $params{$_} and $self->{$_} = delete $params{$_} for qw( exit_on_die max_calls );
+   foreach (qw( exit_on_die )) {
+      $worker->{$_} = delete $params{$_} if exists $params{$_};
+   }
 
-   $self->SUPER::configure( %params );
+   $worker->SUPER::configure( %params );
 }
 
 sub stop
 {
    my $worker = shift;
-   $worker->{arg_channel}->close;
+   $worker->stdin->close;
 
    if( my $function = $worker->parent ) {
-      delete $function->{workers}{$worker->id};
+      delete $function->{workers}{$worker->pid};
    }
 }
 
 sub call
 {
    my $worker = shift;
-   my ( $type, $args, $future ) = @_;
+   my ( $request, $on_result ) = @_;
 
-   if( $type eq "args" ) {
-      $worker->{arg_channel}->send( $args );
-   }
-   elsif( $type eq "frozen" ) {
-      $worker->{arg_channel}->send_frozen( $args );
-   }
-   else {
-      die "TODO: unsure $type\n";
-   }
+   push @{ $worker->{on_result_queue} }, $worker->_capture_weakself( sub {
+      my $worker = shift;
+      my $type = shift;
 
-   $worker->{ret_channel}->recv(
-      on_recv => $worker->_capture_weakself( sub {
-         my ( $worker, $channel, $result ) = @_;
-         my ( $type, @values ) = @$result;
+      $worker->{busy} = 0;
 
-         $worker->{busy} = 0;
+      my $function = $worker->parent;
 
-         my $function = $worker->parent;
-
-         if( $type eq "r" ) {
-            $future->done( @values );
-         }
-         elsif( $type eq "e" ) {
-            $future->fail( $values[0], @values );
-            $worker->stop if $worker->{exit_on_die};
-         }
-         else {
-            die "Unrecognised type from worker - $type\n";
-         }
-
-         $worker->stop if !$worker->{max_calls};
-
-         $function->_dispatch_pending if $function;
-      } ),
-      on_eof => $worker->_capture_weakself( sub {
-         my ( $worker, $channel ) = @_;
-
-         $worker->{busy} = 0;
-
-         my $function = $worker->parent;
-
-         $future->fail( "closed", "closed" );
+      if( $type eq "eof" ) {
+         $on_result->( error => "closed" );
          $worker->stop;
+      }
+      elsif( $type eq "r" ) {
+         $on_result->( return => @_ );
+      }
+      elsif( $type eq "e" ) {
+         $on_result->( error => @_ );
+         $worker->stop if $worker->{exit_on_die};
+      }
+      else {
+         die "Unrecognised type from worker - $type\n";
+      }
 
-         $function->_dispatch_pending if $function;
-      } ),
-   );
+      $function->_dispatch_pending if $function;
+   } );
 
+   $worker->stdin->write( pack("I", length $request) . $request );
    $worker->{busy} = 1;
-   $worker->{max_calls}--;
 }
 
+# Keep perl happy; keep Britain tidy
+1;
+
+__END__
+
 =head1 NOTES
 
 For the record, 123454321 is 11111 * 11111, a square number, and therefore not
@@ -643,7 +614,3 @@ prime.
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
