@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 use Carp;
 
@@ -223,9 +223,7 @@ sub _watch_read
    my $fh = $self->read_handle or return;
 
    if( $want ) {
-      $self->{cb_r} ||= $self->{on_read_ready} ?
-         $self->_capture_weakself( $self->{on_read_ready} ) :
-         $self->_capture_weakself( 'on_read_ready' );
+      $self->{cb_r} ||= $self->make_event_cb( 'on_read_ready' );
 
       $loop->watch_io(
          handle => $fh,
@@ -249,9 +247,7 @@ sub _watch_write
    my $fh = $self->write_handle or return;
 
    if( $want ) {
-      $self->{cb_w} ||= $self->{on_write_ready} ?
-         $self->_capture_weakself( $self->{on_write_ready} ) :
-         $self->_capture_weakself( 'on_write_ready' );
+      $self->{cb_w} ||= $self->make_event_cb( 'on_write_ready' );
 
       $loop->watch_io(
          handle => $fh,
@@ -356,7 +352,9 @@ sub close
 
 Closes the underlying read or write handle, and deconfigures it from the
 object. Neither of these methods will invoke the C<on_closed> event, nor
-remove the object from the Loop.
+remove the object from the Loop if there is still one open handle in the
+object. Only when both handles are closed, will C<on_closed> be fired, and the
+object removed.
 
 =cut
 
